<?php
/**
 * @package SJ Responsive Listing for Zoo
 * @version 1.0.0
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @copyright (c) 2014 YouTech Company. All Rights Reserved.
 * @author YouTech Company http://www.smartaddons.com
 */

defined('_JEXEC') or die;

include_once dirname(__FILE__) . '/helper_base.php';

abstract class ZooResponsiveListingHelper extends ZooResponsiveListingBaseHelper
{

	protected static $app = null;
	protected static $application = null;
	public static $total = null;

	public static function getList($params)
	{
		$file_config = JPATH_ADMINISTRATOR . '/components/com_zoo/config.php';
		if (!file_exists($file_config)) return;
		require_once($file_config);
		$list = array();
		$_list = array();
		self::$app = $_app = App::getInstance('zoo');
		$_app->system->language->load('com_zoo');
		$application_id = $params->get('zoo_app', 0);
		self::$application = $_application = $_app->table->application->get($application_id);
		$category_ids = $params->get('catid');
		settype($category_ids, 'array');
		if (empty($category_ids)) return;
		$catids = self::_getPublishCategories($category_ids);
		if (empty($catids)) return;
		$_list = self::_getCategoriesInfor($catids);
		self::$total = self::_countItems($catids, $params);
		if (empty($_list)) return;
		$categories = array();
		foreach ($_list as $cat) {
			$categories[$cat->id] = $cat;
		}

		$items = self::_getItems($catids, $params);
		if (empty($items)) return;
		foreach ($items as $item) {
			$category = $categories[$item->category_id];
			if (isset($category->count)) {
				$category->count++;
			} else {
				$category->count = 1;
			}
		}

		if ($params->get('tab_all_display', 1)) {
			$all = new stdClass();
			$all->id = '*';
			$all->count = count($items);
			$all->title = JText::_('ALL_LABEL');
			array_unshift($categories, $all);
		}

		$selected = false;
		$catidpreload = (int)$params->get('catid_preload');
		foreach ($categories as $cat) {
			if (isset($cat->sel)) {
				unset($cat->sel);
			}
			if ($cat->id == $catidpreload && $cat->count > 0) {
				$cat->sel = 'sel';
				$selected = true;
			}
		}

		// first tab is active
		if (!$selected) {
			foreach ($categories as $cat) {
				if ($cat->count > 0) {
					$cat->sel = 'sel';
					break;
				}
			}
		}

		$list['categories'] = $categories;

		$list['items'] = $items;

		return $list;
	}

	/**
	 * Process getCategory Information
	 **/

	private static function _getCategoriesInfor($catids = array(), $cat_order_by = null)
	{
		$list = array();
		$categories = array();
		!is_array($catids) && settype($catids, 'array');
		foreach ($catids as $catid) {
			$categories[$catid] = self::$app->table->category->get($catid);
		}
		if (empty($categories)) return;
		foreach ($categories as $key => $category) {
			$category->_description = self::_cleanText($category->description);
			$category->link = self::$app->route->category($category);
			$category->title = $category->name;
			$category->count = 0;

			$list[$key] = $category;
			if ($cat_order_by) {
				switch ($cat_order_by) {
					default:
					case 'title':
						usort($list, create_function('$a, $b', 'return strnatcasecmp( $a->name, $b->name);'));
						break;
					case 'lft':
						usort($list, create_function('$a, $b', 'return $a->ordering < $b->ordering;'));
						break;
					case 'random':
						!is_array($list) && settype($list, 'array');
						shuffle($list);
						break;
				}
			}

		}
		return $list;
	}

	/**
	 * Process check category publish
	 **/

	private static function _getPublishCategories($catids = array())
	{
		$_catids = array();
		foreach ($catids as $catid) {
			$ispublished = self::$app->table->category->get($catid)->isPublished();
			if ($ispublished >= 1) {
				$_catids[] = $catid;
			}
		}
		return $_catids;
	}

	/**
	 * Process getItems
	 **/

	private static function _getItems($catids, $params)
	{
		// if ($params->get('show_child_category_articles', 0) && (int)$params->get('levels', 0) > 0) {
		// $levels = $params->get('levels', 1) ? $params->get('levels', 1) : 9999;
		// $catids = self::_getChildrenCategory($catids, $levels);
		// }
		!is_array($catids) && settype($catids, 'array');
		$items = self::_getItemsDB($catids, $params);
		if (empty($items)) return;
		$list = array();
		foreach ($items as  $item) {
			$categories = self::_getCategoriesInfor($item->category_id);
			foreach( $categories as $cate){
				$item->catlink = $cate->link;
				$item->category_title = $cate->name;
			}
			self::_getElemnts($item, $params);
			$item->link = self::$app->route->item($item);
			$item->title = $item->name;
			$item->displayIntrotext = isset($item->_description) ? $item->_description : '';
			$item->socialbuttons = isset($item->_socialbuttons) ? $item->_socialbuttons : '';
			$item->relateditems = isset($item->_relateditems) ? $item->_relateditems : '';
			$item->relateditems .= isset($item->_extenal_related_articles) ? $item->_extenal_related_articles : '';
			$item->itemtag = isset($item->_itemtag) ? $item->_itemtag : '';
			//$item->primarycatid = $item->getPrimaryCategoryId();
			$list[] = $item;
		}
		return $list;
	}

	/**
	 * Process getElements
	 **/

	private static function _getElemnts($item, $params)
	{
		$_items = array();
		$elements = array_merge($item->getElements(), $item->getCoreElements());
		foreach ($elements as $id => $element) {
			$data = $element->getElementData();
			$type = $element->getElementType();
			$name = $element->config->get('name');
			switch ($type) {
				case 'text':
					$item->_subtitle = self::_cleanText($data->get('value'));
					$_items[$item->id][$type] = isset($item->_subtitle);
					break;
				case 'textarea':
					$textareas = (array)$item->elements[$id];
					if (!empty($textareas)) {
						foreach ($textareas as $textarea) {
							$text = isset($textarea['value']) ? $textarea['value'] : '';
							if (self::_trimEncode($text) != '' && !isset($item->_description)) {
								$item->_description = self::_cleanText($text);
								$_items[$item->id][$type] = isset($item->_description);
							}
							if (!isset($item->_textarea) && count(self::getInlineImages($text))) {
								$item->_textarea = $text;
								$_items[$item->id][$type] = isset($item->_textarea);
							}
							if (isset($item->_textarea) && isset($item->_description)) {
								break;
							}
						}
					}
					break;
				case 'relateditems':
					$params_cf = array('first' => true,
						'last' => false,
						'showlabel' => '0',
						'altlabel' => '',
						'order' =>
							array(
								0 => '_itemcreated',
								1 => '',
								2 => '',
								3 => ''),
						'layout' => '',
						'link_to_item' => '1',
						'separated_by' => 'tag=[<li>%s</li>]',
						'element' => $id,
						'_layout' => 'full',
						'_position' => 'related',
						'_index' => 0);
					$getitem = $data->get('item', array());
					if (!empty($getitem)) {
						$item->_relateditems = $element->render($params_cf);
						$_items[$item->id][$type] = isset($item->relateditems);
					}
					break;
				case 'socialbuttons':
					$params_cf = array(
						'ggsize' => 'medium',
						'ggannotation' => '0',
						'ggwidth' => '100',
						'fbheight' => '20',
						'fbwidth' => '100',
						'fbshow_faces' => 'false',
						'fblayout' => 'button_count',
						'fbaction' => 'like',
						'fbcolorscheme' => 'light',
						'fbref' => '',
						'twvia' => '',
						'twtext' => '',
						'twrelated' => '',
						'twcount' => '0',
						'element' => $id,
						'_layout' => 'full',
						'_position' => 'bottom',
						'_index' => 0,
						'style' => 'block');
					if ($data->get('value') != '') {
						$item->_socialbuttons = $element->render($params_cf);
						$_items[$item->id][$type] = isset($item->_socialbuttons);
					}
					break;
				case 'link':
					$params_cf = array(
						'first' => false,
						'last' => true,
						'showlabel' => '0',
						'altlabel' => '',
						'separated_by' => 'tag=[<li>%s</li>]',
						'element' => $id,
						'_layout' => 'full',
						'_position' => 'related',
						'_index' => 1);
					if ($data->get('value') != null) {
						$item->_extenal_related_articles = $element->render($params_cf);
						$_items[$item->id][$type] = isset($item->_extenal_related_articles);
					}
					break;
				case 'image':
					$flag = true;
					if (!isset($item->_teaser_image)) {
						$flag = false;
						$item->_teaser_image = $data->get('file');
						$_items[$item->id][$type] = isset($item->_teaser_image);
					}
					if ($flag) {
						$el_name = true;
						$item->_image = $data->get('file');
						$_items[$item->id][$type] = isset($item->_image);
						continue;
					}
					break;
				case 'itemtag':
					$params_cf = array(
						'first' => true,
						'last' => true,
						'showlabel' => '1',
						'altlabel' => '',
						'linked' => '1',
						'separated_by' => 'separator=[  |  ]',
						'element' => '_itemtag',
						'_layout' => 'full',
						'_position' => 'taxonomy',
						'_index' => 0,
						'style' => 'list');
					$label = '';
					if (isset($params_cf['showlabel']) && $params_cf['showlabel']) {
						$label .= '<span >';
						$label .= ($params_cf['altlabel']) ? $params_cf['altlabel'] : $element->config->get('name');
						$label .= ': </span>';
					}
					$tags = (array)$item->getTags();
					if (!empty($tags)) {
						$item->_itemtag = ($element->render($params_cf)) ? $label . $element->render($params_cf) : '';
						$_items[$item->id][$type] = isset($item->_itemtag);
					}
					break;
				default:
					break;
			}
		}
		return $_items;
	}

	/**
	 * Process getChildCatgory for depth level
	 **/

	private static function _getChildrenCategory($catids, $levels, $onlychild = false)
	{
		$_categories = self::$application->getCategoryTree(true);
		$additional_catids = array();
		settype($catids, 'array');
		if (empty($catids)) return;
		if ($levels > 0) {
			foreach ($catids as $catid) {
				if (($categories = $_categories[$catid]) && $categories->hasChildren()) {
					$recursive = true;
					$items = $categories->getChildren($recursive);
					if ($items) {
						foreach ($items as $category) {
							$path = array_reverse($category->getPath());
							$depth = count(array_slice($path, array_search($catid, $path))) - 1;
							if ($levels && $levels < $depth) {
								continue;
							}
							$parent = $category->hasChildren() && !($levels && $levels < $depth + 1);
							$condition = (($depth - $parent) <= $levels);
							if ($condition) {
								$additional_catids[] = $category->id;
							}
						}
					}
				}
			}
		}
		if ($onlychild) {
			$catids = array_unique($additional_catids);
		} else {
			$catids = array_unique(array_merge($catids, $additional_catids));
		}

		return $catids;
	}

	/**
	 * Process getItems form database
	 **/

	private static function _getItemsDB($category_id, $params)
	{
		$app = self::$app;
		$db = $app->database;
		$date = $app->date->create();
		$now = $db->Quote($date->toSQL());
		$null = $db->Quote($db->getNullDate());
		$application_id = $params->get('zoo_app', 0);
		$published = true;
		$limit = $params->get('count');
		$app = JFactory::getApplication();
		$appParams = $app->getParams();
		$offset = $app->input->getInt('ajax_reslisting_start', 0);
		$query = "SELECT a.*, b.category_id, EXISTS (SELECT TRUE FROM " . ZOO_TABLE_CATEGORY_ITEM . " WHERE item_id=a.id AND 	category_id=0) AS frontpage"
			. " FROM " . ZOO_TABLE_ITEM . " AS a"
			. " LEFT JOIN " . ZOO_TABLE_CATEGORY_ITEM . " AS b ON a.id = b.item_id"
			. " WHERE a.application_id = " . (int)$application_id
			. " AND a" . self::_getDBAccessString()
			. ($published == true ? " AND a.state = 1"
				. " AND (a.publish_up = " . $null . " OR a.publish_up <= " . $now . ")"
				. " AND (a.publish_down = " . $null . " OR a.publish_down >= " . $now . ")" : "")
			. " AND b.category_id " . (is_array($category_id) ? " IN (" . implode(",", $category_id) . ")" : " = " . (int)$category_id)
			. " GROUP BY a.id " . self::_itemFilter($params)
			. " ORDER BY " . self::_itemOrders($params)
			. (($limit ? " LIMIT " . (int)$offset . "," . (int)$limit : ""));
		return self::_queryObjectList($query);
	}


	private static function _countItems($category_id, $params)
	{
		$app = self::$app;
		$db = $app->database;
		$date = $app->date->create();
		$now = $db->Quote($date->toSQL());
		$null = $db->Quote($db->getNullDate());
		$application_id = $params->get('zoo_app', 0);
		$published = true;
		$offset = 0;
		$limit = $params->get('count');
		$query = "SELECT a.id, EXISTS (SELECT TRUE FROM " . ZOO_TABLE_CATEGORY_ITEM . " WHERE item_id=a.id AND 	category_id=0) AS frontpage"
			. " FROM " . ZOO_TABLE_ITEM . " AS a"
			. " LEFT JOIN " . ZOO_TABLE_CATEGORY_ITEM . " AS b ON a.id = b.item_id"
			. " WHERE a.application_id = " . (int)$application_id
			. " AND a" . self::_getDBAccessString()
			. ($published == true ? " AND a.state = 1"
				. " AND (a.publish_up = " . $null . " OR a.publish_up <= " . $now . ")"
				. " AND (a.publish_down = " . $null . " OR a.publish_down >= " . $now . ")" : "")
			. " AND b.category_id " . (is_array($category_id) ? " IN (" . implode(",", $category_id) . ")" : " = " . (int)$category_id)
			. " GROUP BY a.id " . self::_itemFilter($params)
			. " ORDER BY " . self::_itemOrders($params);

		return count(self::_queryObjectList($query));
	}

	/**
	 * Process queryObjectList
	 **/

	private static function _queryObjectList($query)
	{
		$app = self::$app;
		$result = $app->database->query($query);
		$objects = array();
		while ($object = $app->database->fetchObject($result, 'Item')) {
			$objects[$object->id] = self::_initObject($object);
		}
		$app->database->freeResult($result);
		return $objects;
	}

	/**
	 * Process initObject
	 **/

	private static function _initObject($object)
	{
		if (property_exists($object, 'app')) {
			$object->app = self::$app;
		}
		return $object;
	}

	/**
	 * Process filter
	 **/

	private static function _itemFilter($params, $alias = 'a')
	{
		$join_filter = "";
		if ($params->get('show_frontpage') != '') {
			switch ($params->get('show_frontpage')) {
				default:
				case '0':
					$join_filter = "";
					break;
				case '1':
					$join_filter = "HAVING frontpage=0";
					break;
				case '2':
					$join_filter = "HAVING frontpage=1";
					break;
			}
		}
		return $join_filter;
	}

	/**
	 * Process order
	 **/

	private static function _itemOrders($params, $alias = 'a')
	{
		$item_order_by = "$alias.priority";
		$order_direction = $params->get('article_ordering_direction', "DESC");
		if ($params->get('article_ordering') != '') {
			$string_order_by = trim($params->get('article_ordering'));
			switch ($string_order_by) {
				default:
				case 'ordering':
					$item_order_by = "$alias.priority " . $order_direction;
					break;
				case 'mostview':
				case 'hits':
					$item_order_by = "$alias.hits " . $order_direction;
					break;
				case 'recently_add':
				case 'created':
					$item_order_by = "$alias.created " . $order_direction;
					break;
				case 'recently_mod':
				case 'modified':
					$item_order_by = "$alias.modified " . $order_direction;
					break;
				case 'title':
					$item_order_by = "$alias.name " . $order_direction;
					break;
				case 'random':
					$item_order_by = "$alias.hits " . $order_direction;
					break;
			}
		}
		return $item_order_by;
	}

	/**
	 * Process acess database
	 **/

	private static function _getDBAccessString()
	{
		$condition = false;
		$app = JFactory::getApplication();
		$params = $app->getParams();
		if ($params instanceof JRegistry && !$params->get('show_noauth', 0)) {
			$user = JFactory::getUser();
			$condition = '.access IN (' . implode(',', $user->getAuthorisedViewLevels()) . ')';
		}
		return $condition;
	}

}
